/****************************************************************************
**
** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Nokia Corporation and its Subsidiary(-ies) nor
**     the names of its contributors may be used to endorse or promote
**     products derived from this software without specific prior written
**     permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QDebug>

#include <QAudioFormat>
#include <QAudioDeviceInfo>

#include "audiosynth.h"

const int DurationSeconds = 1;
const int ToneFrequencyHz = 600;
const int DataFrequencyHz = 44100;
const int BufferSize      = 32768;

Synthesizer::Synthesizer(QObject *parent)    :   
	QObject(parent),	
	m_generator(0),
        m_audioOutput(0),
        m_output(0),
        m_buffer(BufferSize, 0)
{
}

void Synthesizer::initializeAudio()
{
    m_format.setFrequency(DataFrequencyHz);
    m_format.setChannels(1);
    m_format.setSampleSize(16);
    m_format.setCodec("audio/pcm");
    m_format.setByteOrder(QAudioFormat::LittleEndian);
    m_format.setSampleType(QAudioFormat::SignedInt);

    QAudioDeviceInfo info(QAudioDeviceInfo::defaultOutputDevice());
    QList<QAudioDeviceInfo> devices = QAudioDeviceInfo::availableDevices(QAudio::AudioOutput);
    for (QList<QAudioDeviceInfo>::iterator s = devices.begin(); s != devices.end(); ++s)
    {
        info = *s;
    
        qDebug() << "trying: " << info.deviceName() << endl;

        if (info.isNull()) {
            qDebug() << "invalid audio dev info" << endl;
            continue;
        }
        if (info.deviceName() == "null") {
            qDebug() << "NULL audio dev info" << endl;
            continue;
        }
        if (!info.isFormatSupported(m_format)) {
	        qWarning() << "Default format not supported - trying to use nearest";
	        m_format = info.nearestFormat(m_format);
    	}	
	m_generator = new Generator(m_format, DurationSeconds*1000000, ToneFrequencyHz, this);
	m_device = info;
    	createAudioOutput();
	qWarning() << __PRETTY_FUNCTION__ << "working ...";
    	break;
    }
}

void Synthesizer::createAudioOutput()
{
    delete m_audioOutput;
    m_audioOutput = 0;
    m_audioOutput = new QAudioOutput(m_device, m_format, this);
    connect(m_audioOutput, SIGNAL(notify()), SLOT(notified()));
    connect(m_audioOutput, SIGNAL(stateChanged(QAudio::State)), SLOT(stateChanged(QAudio::State)));
    m_generator->start();
    m_audioOutput->start(m_generator);
    qWarning() << __PRETTY_FUNCTION__ << "created ...";
}

Synthesizer::~Synthesizer()
{

}

void Synthesizer::notified()
{
    qWarning() << "bytesFree = " << m_audioOutput->bytesFree()
               << ", " << "elapsedUSecs = " << m_audioOutput->elapsedUSecs()
               << ", " << "processedUSecs = " << m_audioOutput->processedUSecs();
}

void Synthesizer::stop()
{
    m_generator->stop();
    m_audioOutput->stop();
    m_audioOutput->disconnect(this);
}

void Synthesizer::start()
{
	qWarning() << __PRETTY_FUNCTION__ << "...";
	initializeAudio();
}

void Synthesizer::stateChanged(QAudio::State state)
{
     switch (state) {
     case QAudio::StoppedState:
         qWarning() << "stopped state ..." << endl;
         if (m_audioOutput->error() != QAudio::NoError) {
             qWarning() << "finished with error " << m_audioOutput->error() << endl;
         } else {
             qWarning() << "completed ..." << endl;
         }
         emit stopped();
        break;
     case QAudio::ActiveState:
         qWarning() << "active state ..." << endl;
         break;
     case QAudio::SuspendedState:
         qWarning() << "suspended state ..." << endl;
         break;
     case QAudio::IdleState:
         qWarning() << "idle state ..." << endl;
         break;
     default:
         qWarning() << "unknown state ..." << endl;
     };
}

